"""Assures API stability against the first released version of the filter API.

The script loops through 'all_tables' and queries it for an object. The output
of that object is ignored, but the columns are compared to the ones stored in
'cache'.

The 'cache' file is checked into git and should never be modified (but if it
is, it will show in the git log). The cache was generated by running python
filter_spec.py --freeze"""

from op5helpers import *
import json
import os
import requests
import subprocess
import sys
import time

query = api_endpoint + 'api/filter/query?query=[%s] all'
cache = os.path.abspath(os.path.dirname(__file__)+'/filter_spec.json')
all_tables = ['hosts', 'services', 'hostgroups', 'servicegroups', 'comments',
'downtimes', 'contacts', 'contactgroups', 'notifications', 'timeperiods',
'saved_filters', 'recurring_downtimes', 'commands']

def setup():
    def _command(cmd, args):
        r = requests.post(api_endpoint + 'api/command/' + cmd, headers={'content-type': 'application/json'}, verify=False, auth=authed_user, data=json.dumps(args))
        eq_(200, r.status_code, "Should have been able to send a command, got %d:\n%s" % (r.status_code, r.text))
        ok_("failed" not in r.text, "Should have been able to send a command, got %d:\n%s" % (r.status_code, r.text))
        return r

    # host + service
    c = Crud('host', 'goggles')
    c.post({
        'host_name': 'goggles',
        'address': 'not-a-proper-host.nowhere.org',
        'active_checks_enabled': True,
        'services': [{
            'service_description': 'edict',
            'check_command': 'check-host-alive',
            'active_checks_enabled': True,
            'check_interval' : 1
            }]
        })

    # hostgroup
    c = Crud('hostgroup', 'jor-el')
    c.post({
        'hostgroup_name': 'jor-el'
        })

    # servicegroup
    c = Crud('servicegroup', 'kal-el')
    c.post({
        'servicegroup_name': 'kal-el'
        })

    # contact
    c = Crud('contact', 'mads')
    c.post({
        'contact_name': 'mads',
        'alias': 'mads'
        })

    # contactgroup
    c = Crud('contactgroup', 'mikkelsen')
    c.post({
        'contactgroup_name': 'mikkelsen',
        'alias': 'mikkelsen'
        })

    # saved_filter TODO proper API?
    # recurring_downtime TODO proper API?
    # notification TODO proper API?
    # comment TODO proper API?

    Change.post()

    # downtime
    _command('SCHEDULE_HOST_DOWNTIME', {
        "host_name": "goggles",
        "start_time": int(time.time()) + 40,
        "end_time": int(time.time()) + 50,
        "fixed": 1,
        "trigger_id": 0,
        "duration": 0,
        "comment": "Theyre in the mountains"
        })

def teardown():
    crud = Crud('host', 'goggles', clean=True)
    crud = Crud('hostgroup', 'jor-el', clean=True)
    crud = Crud('servicegroup', 'kal-el', clean=True)
    crud = Crud('contact', 'mads', clean=True)
    crud = Crud('contactgroup', 'mikkelsen', clean=True)
    Change.post()

def _query_table(table):
    r = requests.get(query % (table,), auth=authed_user, params={'limit': 1}, verify=False)
    eq_(200, r.status_code, "Got %d for %s:\n%s" % (r.status_code, r.url, r.text))
    try:
        return r.json()[0]
    except IndexError:
        raise Exception("Got bad (or empty) json from %s:\n\n%s" % (r.url, r.text))

def test_columns():
    real_columns = json.load(open(cache))
    for table in real_columns:
        row = _query_table(table)
        """ let's be lazy and only check a single level,
        that way we don't have to know the difference between the
        expected formatting differences between host.service and
        host.services_with_info, for example """
        perceived = set(row.keys())

        """ see
        https://nose.readthedocs.org/en/latest/writing_tests.html#test-generators
        and https://github.com/nose-devs/nose/issues/69 """
        cc = _check_columns
        cc.description = "test_columns_for_table_" + table
        yield cc, set(real_columns[table]), perceived, table

def _check_columns(columns_from_spec, columns_from_current_api, table):
    excessive = columns_from_current_api - columns_from_spec
    missing = columns_from_spec - columns_from_current_api
    ok_(not missing and not excessive, "These columns should be in in the %s table but they were missing from the current revision:\n%s\n\nand these columns aren't specified in the frozen version of the API:\n%s" % (table, "\n".join(missing), "\n".join(excessive)))

def _freeze():
    if os.path.isfile(cache):
        raise Exception("You really should not freeze a new cache file.. like, ever")
    result = {}
    for table in all_tables:
        result[table] = _query_table(table).keys()
    with open(cache, 'w') as fp:
        json.dump(result, fp)

if __name__ == '__main__':
    if '--freeze' in sys.argv:
        if not 'abracadabra' in sys.argv:
            print("You really shouldn't try to freeze stuff without providing " +
                    "the secret password. Shame. On. You.")
            sys.exit(1)
        try:
            setup()
            _freeze()
        finally:
            teardown()
        sys.exit(0)
    print("What *are* you trying to do? --freeze ?")
    sys.exit(1)
